@@ -19,9 +19,10 @@ pub struct TerminalConfig {
         desc = "Set the terminal line height, If 0, it uses editor line height"
     )]
     pub line_height: f64,
-    #[field_names(desc = "Profiles available in terminal pane")]
+
+    #[field_names(skip)]
     pub profiles: HashMap<String, TerminalProfile>,
-    #[field_names(desc = "Default profile for each platform")]
+    #[field_names(skip)]
     pub default_profile: HashMap<String, String>,
 
     #[serde(skip)]
@@ -78,7 +79,7 @@ impl TerminalConfig {
     ) -> Option<lapce_rpc::terminal::TerminalProfile> {
         let profile = self.profiles.get(
             self.default_profile
-                .get(&std::env::consts::OS.to_string())
+                .get(std::env::consts::OS)
                 .unwrap_or(&String::from("default")),
         )?;
         let workdir = if let Some(workdir) = &profile.workdir {
