@@ -14,11 +14,11 @@ use std::{
 
 use floem::{
     action::exec_after,
-    cosmic_text::{Attrs, AttrsList, FamilyOwned, TextLayout},
     ext_event::create_ext_action,
     keyboard::Modifiers,
     peniko::Color,
     reactive::{batch, ReadSignal, RwSignal, Scope},
+    text::{Attrs, AttrsList, FamilyOwned, TextLayout},
     views::editor::{
         actions::CommonAction,
         command::{Command, CommandExecuted},
@@ -29,6 +29,7 @@ use floem::{
         view::{ScreenLines, ScreenLinesBase},
         CursorInfo, Editor, EditorStyle,
     },
+    ViewId,
 };
 use itertools::Itertools;
 use lapce_core::{
@@ -63,7 +64,8 @@ use lapce_xi_rope::{
     Interval, Rope, RopeDelta, Transformer,
 };
 use lsp_types::{
-    CodeActionResponse, Diagnostic, DiagnosticSeverity, InlayHint, InlayHintLabel,
+    CodeActionOrCommand, CodeLens, Diagnostic, DiagnosticSeverity, InlayHint,
+    InlayHintLabel,
 };
 use serde::{Deserialize, Serialize};
 use smallvec::SmallVec;
@@ -149,7 +151,10 @@ pub struct DocInfo {
 }
 
 /// (Offset -> (Plugin the code actions are from, Code Actions))
-pub type CodeActions = im::HashMap<usize, Arc<(PluginId, CodeActionResponse)>>;
+pub type CodeActions =
+    im::HashMap<usize, (PluginId, im::Vector<CodeActionOrCommand>)>;
+
+pub type AllCodeLens = im::HashMap<usize, (PluginId, usize, im::Vector<CodeLens>)>;
 
 #[derive(Clone)]
 pub struct Doc {
@@ -179,6 +184,8 @@ pub struct Doc {
     /// (Offset -> (Plugin the code actions are from, Code Actions))
     pub code_actions: RwSignal<CodeActions>,
 
+    pub code_lens: RwSignal<AllCodeLens>,
+
     /// Stores information about different versions of the document from source control.
     histories: RwSignal<im::HashMap<String, DocumentHistory>>,
     pub head_changes: RwSignal<im::Vector<DiffLines>>,
@@ -241,6 +248,7 @@ impl Doc {
             preedit: PreeditData::new(cx),
             editors,
             common,
+            code_lens: cx.create_rw_signal(im::HashMap::new()),
         }
     }
 
@@ -289,6 +297,7 @@ impl Doc {
             preedit: PreeditData::new(cx),
             editors,
             common,
+            code_lens: cx.create_rw_signal(im::HashMap::new()),
         }
     }
 
@@ -337,6 +346,7 @@ impl Doc {
             preedit: PreeditData::new(cx),
             editors,
             common,
+            code_lens: cx.create_rw_signal(im::HashMap::new()),
         }
     }
 
@@ -629,6 +639,7 @@ impl Doc {
             self.do_bracket_colorization();
             self.clear_code_actions();
             self.clear_style_cache();
+            self.get_code_lens();
         });
     }
 
@@ -791,7 +802,7 @@ impl Doc {
     }
 
     /// Request semantic styles for the buffer from the LSP through the proxy.
-    fn get_semantic_styles(&self) {
+    pub fn get_semantic_styles(&self) {
         if !self.loaded() {
             return;
         }
@@ -803,24 +814,37 @@ impl Doc {
                 return;
             };
 
-        let (rev, len) = self.buffer.with_untracked(|b| (b.rev(), b.len()));
-
-        let syntactic_styles =
-            self.syntax.with_untracked(|syntax| syntax.styles.clone());
+        let (atomic_rev, rev, len) = self
+            .buffer
+            .with_untracked(|b| (b.atomic_rev(), b.rev(), b.len()));
 
         let doc = self.clone();
         let send = create_ext_action(self.scope, move |styles| {
-            if doc.buffer.with_untracked(|b| b.rev()) == rev {
-                doc.semantic_styles.set(Some(styles));
-                doc.clear_style_cache();
+            if let Some(styles) = styles {
+                if doc.buffer.with_untracked(|b| b.rev()) == rev {
+                    doc.semantic_styles.set(Some(styles));
+                    doc.clear_style_cache();
+                }
             }
         });
 
         self.common.proxy.get_semantic_tokens(path, move |result| {
             if let Ok(ProxyResponse::GetSemanticTokens { styles }) = result {
-                rayon::spawn(move || {
+                if styles.styles.is_empty() {
+                    send(None);
+                    return;
+                }
+                if atomic_rev.load(atomic::Ordering::Acquire) != rev {
+                    send(None);
+                    return;
+                }
+                std::thread::spawn(move || {
                     let mut styles_span = SpansBuilder::new(len);
                     for style in styles.styles {
+                        if atomic_rev.load(atomic::Ordering::Acquire) != rev {
+                            send(None);
+                            return;
+                        }
                         styles_span.add_span(
                             Interval::new(style.start, style.end),
                             style.style,
@@ -828,24 +852,58 @@ impl Doc {
                     }
 
                     let styles = styles_span.build();
-
-                    let styles = if let Some(syntactic_styles) = syntactic_styles {
-                        syntactic_styles.merge(&styles, |a, b| {
-                            if let Some(b) = b {
-                                return b.clone();
-                            }
-                            a.clone()
-                        })
-                    } else {
-                        styles
-                    };
-
-                    send(styles);
+                    send(Some(styles));
                 });
+            } else {
+                send(None);
             }
         });
     }
 
+    pub fn get_code_lens(&self) {
+        let cx = self.scope;
+        let doc = self.clone();
+        self.code_lens.update(|code_lens| {
+            code_lens.clear();
+        });
+        let rev = self.rev();
+        if let DocContent::File { path, .. } = doc.content.get_untracked() {
+            let send = create_ext_action(cx, move |result| {
+                if rev != doc.rev() {
+                    return;
+                }
+                if let Ok(ProxyResponse::GetCodeLensResponse { plugin_id, resp }) =
+                    result
+                {
+                    let Some(codelens) = resp else {
+                        return;
+                    };
+                    doc.code_lens.update(|code_lens| {
+                        for codelens in codelens {
+                            let entry = code_lens
+                                .entry(codelens.range.start.line as usize)
+                                .or_insert_with(|| {
+                                    (
+                                        plugin_id,
+                                        doc.buffer.with_untracked(|b| {
+                                            b.offset_of_line(
+                                                codelens.range.start.line as usize,
+                                            )
+                                        }),
+                                        im::Vector::new(),
+                                    )
+                                });
+                            entry.2.push_back(codelens);
+                        }
+                    });
+                }
+            });
+            self.common.proxy.get_code_lens(path, move |result| {
+                send(result);
+            });
+        }
+    }
+
     /// Request inlay hints for the buffer from the LSP through the proxy.
     fn get_inlay_hints(&self) {
         if !self.loaded() {
@@ -1571,7 +1629,7 @@ impl DocumentPhantom for Doc {
                                     Some(PhantomText {
                                         kind: PhantomTextKind::Diagnostic,
                                         col: end_offset - start_offset,
-                                        affinity: None,
+                                        affinity: Some(CursorAffinity::Backward),
                                         text,
                                         fg: Some(fg),
                                         font_size: Some(
@@ -1753,23 +1811,23 @@ impl Styling for DocStyling {
         &self,
         _: EditorId,
         _line: usize,
-    ) -> std::borrow::Cow<[floem::cosmic_text::FamilyOwned]> {
+    ) -> std::borrow::Cow<[floem::text::FamilyOwned]> {
         // TODO: cache this
         Cow::Owned(self.config.with_untracked(|config| {
             FamilyOwned::parse_list(&config.editor.font_family).collect()
         }))
     }
 
-    fn weight(&self, _: EditorId, _line: usize) -> floem::cosmic_text::Weight {
-        floem::cosmic_text::Weight::NORMAL
+    fn weight(&self, _: EditorId, _line: usize) -> floem::text::Weight {
+        floem::text::Weight::NORMAL
     }
 
-    fn italic_style(&self, _: EditorId, _line: usize) -> floem::cosmic_text::Style {
-        floem::cosmic_text::Style::Normal
+    fn italic_style(&self, _: EditorId, _line: usize) -> floem::text::Style {
+        floem::text::Style::Normal
     }
 
-    fn stretch(&self, _: EditorId, _line: usize) -> floem::cosmic_text::Stretch {
-        floem::cosmic_text::Stretch::Normal
+    fn stretch(&self, _: EditorId, _line: usize) -> floem::text::Stretch {
+        floem::text::Stretch::Normal
     }
 
     fn indent_line(&self, _: EditorId, line: usize, line_content: &str) -> usize {
@@ -1974,7 +2032,7 @@ fn syntax_prev_unmatched(
 
 fn should_blink(
     focus: RwSignal<Focus>,
-    keyboard_focus: RwSignal<Option<floem::id::Id>>,
+    keyboard_focus: RwSignal<Option<ViewId>>,
 ) -> impl Fn() -> bool {
     move || {
         let Some(focus) = focus.try_get_untracked() else {
@@ -2033,8 +2091,8 @@ fn extra_styles_for_range(
                 return None;
             }
 
-            let height = (run.glyph_ascent + run.glyph_descent) as f64;
-            let y = run.line_y as f64 - run.glyph_ascent as f64;
+            let height = (run.max_ascent + run.max_descent) as f64;
+            let y = run.line_y as f64 - run.max_ascent as f64;
 
             Some(LineExtraStyle {
                 x,
