@@ -15,6 +15,7 @@ pub enum PanelKind {
     Search,
     Problem,
     Debug,
+    CallHierarchy,
 }
 
 impl PanelKind {
@@ -27,6 +28,7 @@ impl PanelKind {
             PanelKind::Search => LapceIcons::SEARCH,
             PanelKind::Problem => LapceIcons::PROBLEM,
             PanelKind::Debug => LapceIcons::DEBUG,
+            PanelKind::CallHierarchy => LapceIcons::LINK,
         }
     }
 
