@@ -15,16 +15,20 @@ use floem::{
         create_effect, create_memo, create_rw_signal, use_context, RwSignal, Scope,
     },
     style::CursorStyle,
-    view::View,
     views::{
         container, dyn_container, dyn_stack, empty, img, label, rich_text, scroll,
         stack, svg, text, Decorators,
     },
+    IntoView, View,
 };
 use indexmap::IndexMap;
-use lapce_core::{directory::Directory, mode::Mode};
+use lapce_core::{command::EditCommand, directory::Directory, mode::Mode};
 use lapce_proxy::plugin::{download_volt, volt_icon, wasi::find_all_volts};
-use lapce_rpc::plugin::{VoltID, VoltInfo, VoltMetadata};
+use lapce_rpc::{
+    core::{CoreNotification, CoreRpcHandler},
+    plugin::{VoltID, VoltInfo, VoltMetadata},
+};
+use lsp_types::MessageType;
 use serde::{Deserialize, Serialize};
 use sha2::{Digest, Sha256};
 
@@ -85,7 +89,7 @@ pub struct AvailableVoltData {
     pub installing: RwSignal<bool>,
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct AvailableVoltList {
     pub loading: RwSignal<bool>,
     pub query_id: RwSignal<usize>,
@@ -94,7 +98,7 @@ pub struct AvailableVoltList {
     pub total: RwSignal<usize>,
 }
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct PluginData {
     pub installed: RwSignal<IndexMap<VoltID, InstalledVoltData>>,
     pub available: AvailableVoltList,
@@ -126,6 +130,14 @@ impl KeyPressFocus for PluginData {
             CommandKind::Edit(_)
             | CommandKind::Move(_)
             | CommandKind::MultiSelection(_) => {
+                #[allow(clippy::single_match)]
+                match command.kind {
+                    CommandKind::Edit(EditCommand::InsertNewLine) => {
+                        return CommandExecuted::Yes
+                    }
+                    _ => {}
+                }
+
                 return self
                     .available
                     .query_editor
@@ -148,6 +160,7 @@ impl PluginData {
         workspace_disabled: HashSet<VoltID>,
         editors: Editors,
         common: Rc<CommonData>,
+        core_rpc: CoreRpcHandler,
     ) -> Self {
         let installed = cx.create_rw_signal(IndexMap::new());
         let available = AvailableVoltList {
@@ -169,7 +182,7 @@ impl PluginData {
             common,
         };
 
-        plugin.load_available_volts("", 0);
+        plugin.load_available_volts("", 0, core_rpc.clone());
 
         {
             let plugin = plugin.clone();
@@ -216,7 +229,7 @@ impl PluginData {
                 plugin.available.query_id.update(|id| *id += 1);
                 plugin.available.loading.set(false);
                 plugin.available.volts.update(|v| v.clear());
-                plugin.load_available_volts(&query, 0);
+                plugin.load_available_volts(&query, 0, core_rpc.clone());
                 query
             });
         }
@@ -278,8 +291,9 @@ impl PluginData {
                 }
             });
             std::thread::spawn(move || {
-                let info: Option<VoltInfo> =
-                    reqwest::blocking::get(url).ok().and_then(|r| r.json().ok());
+                let info: Option<VoltInfo> = lapce_proxy::get_url(url, None)
+                    .ok()
+                    .and_then(|r| r.json().ok());
                 send(info);
             });
         }
@@ -316,7 +330,12 @@ impl PluginData {
         }
     }
 
-    fn load_available_volts(&self, query: &str, offset: usize) {
+    fn load_available_volts(
+        &self,
+        query: &str,
+        offset: usize,
+        core_rpc: CoreRpcHandler,
+    ) {
         if self.available.loading.get_untracked() {
             return;
         }
@@ -336,36 +355,48 @@ impl PluginData {
                     return;
                 }
 
-                if let Ok(new) = new {
-                    volts.update(|volts| {
-                        volts.extend(new.plugins.into_iter().map(|volt| {
-                            let icon = cx.create_rw_signal(None);
-                            let send = create_ext_action(cx, move |result| {
-                                if let Ok(i) = result {
-                                    icon.set(Some(i));
-                                }
-                            });
-                            {
-                                let volt = volt.clone();
-                                std::thread::spawn(move || {
-                                    let result = Self::load_icon(&volt);
-                                    send(result);
+                match new {
+                    Ok(new) => {
+                        volts.update(|volts| {
+                            volts.extend(new.plugins.into_iter().map(|volt| {
+                                let icon = cx.create_rw_signal(None);
+                                let send = create_ext_action(cx, move |result| {
+                                    if let Ok(i) = result {
+                                        icon.set(Some(i));
+                                    }
                                 });
-                            }
+                                {
+                                    let volt = volt.clone();
+                                    std::thread::spawn(move || {
+                                        let result = Self::load_icon(&volt);
+                                        send(result);
+                                    });
+                                }
 
-                            let data = AvailableVoltData {
-                                info: cx.create_rw_signal(volt.clone()),
-                                icon,
-                                installing: cx.create_rw_signal(false),
-                            };
-                            all.update(|all| {
-                                all.insert(volt.id(), data.clone());
-                            });
+                                let data = AvailableVoltData {
+                                    info: cx.create_rw_signal(volt.clone()),
+                                    icon,
+                                    installing: cx.create_rw_signal(false),
+                                };
+                                all.update(|all| {
+                                    all.insert(volt.id(), data.clone());
+                                });
 
-                            (volt.id(), data)
-                        }));
-                    });
-                    volts_total.set(new.total);
+                                (volt.id(), data)
+                            }));
+                        });
+                        volts_total.set(new.total);
+                    }
+                    Err(err) => {
+                        tracing::error!("{:?}", err);
+                        core_rpc.notification(CoreNotification::ShowMessage {
+                            title: "Request Available Plugins".to_string(),
+                            message: lsp_types::ShowMessageParams {
+                                typ: MessageType::ERROR,
+                                message: err.to_string(),
+                            },
+                        });
+                    }
                 }
             });
 
@@ -395,7 +426,7 @@ impl PluginData {
         let content = match cache_content {
             Some(content) => content,
             None => {
-                let resp = reqwest::blocking::get(&url)?;
+                let resp = lapce_proxy::get_url(&url, None)?;
                 if !resp.status().is_success() {
                     return Err(anyhow::anyhow!("can't download icon"));
                 }
@@ -420,7 +451,7 @@ impl PluginData {
             "https://plugins.lapce.dev/api/v1/plugins/{}/{}/{}/readme",
             volt.author, volt.name, volt.version
         );
-        let resp = reqwest::blocking::get(url)?;
+        let resp = lapce_proxy::get_url(&url, None)?;
         if resp.status() != 200 {
             let text = parse_markdown("Plugin doesn't have a README", 2.0, config);
             return Ok(text);
@@ -434,7 +465,7 @@ impl PluginData {
         let url = format!(
             "https://plugins.lapce.dev/api/v1/plugins?q={query}&offset={offset}"
         );
-        let plugins: VoltsInfo = reqwest::blocking::get(url)?.json()?;
+        let plugins: VoltsInfo = lapce_proxy::get_url(url, None)?.json()?;
         Ok(plugins)
     }
 
@@ -443,7 +474,7 @@ impl PluginData {
             >= self.available.total.get_untracked()
     }
 
-    pub fn load_more_available(&self) {
+    pub fn load_more_available(&self, core_rpc: CoreRpcHandler) {
         if self.all_loaded() {
             return;
         }
@@ -455,7 +486,7 @@ impl PluginData {
             .buffer
             .with_untracked(|buffer| buffer.to_string());
         let offset = self.available.volts.with_untracked(|v| v.len());
-        self.load_available_volts(&query, offset);
+        self.load_available_volts(&query, offset, core_rpc);
     }
 
     pub fn install_volt(&self, info: VoltInfo) {
@@ -763,6 +794,7 @@ pub fn plugin_info_view(plugin: PluginData, volt: VoltID) -> impl View {
                         .disabled(|s| {
                             s.background(config.color(LapceColor::EDITOR_DIM))
                         })
+                        .selectable(false)
                 })
                 .disabled(move || installing.map(|i| i.get()).unwrap_or(false))
                 .on_click_stop(move |_| {
@@ -835,8 +867,7 @@ pub fn plugin_info_view(plugin: PluginData, volt: VoltID) -> impl View {
                                 let scroll_width = scroll_width.get();
                                 s.max_width(
                                     scroll_width
-                                        .max(200.0 + 60.0 * 2.0 + 200.0)
-                                        .min(800.0)
+                                        .clamp(200.0 + 60.0 * 2.0 + 200.0, 800.0)
                                         - 60.0 * 2.0
                                         - 200.0,
                                 )
@@ -971,7 +1002,7 @@ pub fn plugin_info_view(plugin: PluginData, volt: VoltID) -> impl View {
                         )
                         .padding(padding)
                 })
-                .any()
+                .into_any()
             },
         )
         .style(|s| s.min_width_full().justify_center()),
@@ -982,4 +1013,5 @@ pub fn plugin_info_view(plugin: PluginData, volt: VoltID) -> impl View {
         }
     })
     .style(|s| s.absolute().size_full())
+    .debug_name("Plugin Info")
 }
