@@ -16,7 +16,7 @@ use crate::{
     window_tab::CommonData,
 };
 
-#[derive(Clone)]
+#[derive(Clone, Debug)]
 pub struct SourceControlData {
     // VCS modified files & whether they should be included in the next commit
     pub file_diffs: RwSignal<IndexMap<PathBuf, (FileDiff, bool)>>,
