@@ -1,44 +1,41 @@
-use std::sync::Arc;
-use std::time::SystemTime;
+use std::{sync::Arc, time::SystemTime};
 
-use alacritty_terminal::index::Side;
-use alacritty_terminal::selection::{Selection, SelectionType};
-use alacritty_terminal::term::search::Match;
-use alacritty_terminal::term::RenderableContent;
 use alacritty_terminal::{
     grid::Dimensions,
-    term::{cell::Flags, test::TermSize},
+    index::Side,
+    selection::{Selection, SelectionType},
+    term::{cell::Flags, test::TermSize, RenderableContent},
 };
-use floem::context::{EventCx, PaintCx};
-use floem::event::Event;
-use floem::peniko::Color;
-use floem::pointer::PointerInputEvent;
-use floem::views::editor::core::register::Clipboard;
-use floem::views::editor::text::SystemClipboard;
 use floem::{
-    cosmic_text::{Attrs, AttrsList, FamilyOwned, TextLayout, Weight},
-    id::Id,
-    peniko::kurbo::{Point, Rect, Size},
+    context::{EventCx, PaintCx},
+    event::{Event, EventPropagation},
+    peniko::{
+        kurbo::{Point, Rect, Size},
+        Color,
+    },
+    pointer::PointerInputEvent,
     reactive::{create_effect, ReadSignal, RwSignal},
-    view::{AnyWidget, View, ViewData, Widget},
-    EventPropagation, Renderer,
+    text::{Attrs, AttrsList, FamilyOwned, TextLayout, Weight},
+    views::editor::{core::register::Clipboard, text::SystemClipboard},
+    Renderer, View, ViewId,
 };
 use lapce_core::mode::Mode;
 use lapce_rpc::{proxy::ProxyRpcHandler, terminal::TermId};
 use lsp_types::Position;
 use parking_lot::RwLock;
+use regex::Regex;
 use unicode_width::UnicodeWidthChar;
 
 use super::{panel::TerminalPanelData, raw::RawTerminal};
-use crate::command::InternalCommand;
-use crate::editor::location::{EditorLocation, EditorPosition};
-use crate::listener::Listener;
-use crate::workspace::LapceWorkspace;
 use crate::{
+    command::InternalCommand,
     config::{color::LapceColor, LapceConfig},
     debug::RunDebugProcess,
+    editor::location::{EditorLocation, EditorPosition},
+    listener::Listener,
     panel::kind::PanelKind,
     window_tab::Focus,
+    workspace::LapceWorkspace,
 };
 
 /// Threshold used for double_click/triple_click.
@@ -59,7 +56,7 @@ struct TerminalLineContent<'a> {
 }
 
 pub struct TerminalView {
-    data: ViewData,
+    id: ViewId,
     term_id: TermId,
     raw: Arc<RwLock<RawTerminal>>,
     mode: ReadSignal<Mode>,
@@ -71,7 +68,7 @@ pub struct TerminalView {
     launch_error: RwSignal<Option<String>>,
     internal_command: Listener<InternalCommand>,
     workspace: Arc<LapceWorkspace>,
-    hyper_matches: Vec<Match>,
+    hyper_regs: Vec<Regex>,
     previous_mouse_action: MouseAction,
     current_mouse_action: MouseAction,
 }
@@ -87,7 +84,7 @@ pub fn terminal_view(
     internal_command: Listener<InternalCommand>,
     workspace: Arc<LapceWorkspace>,
 ) -> TerminalView {
-    let id = Id::next();
+    let id = ViewId::new();
 
     create_effect(move |_| {
         let raw = raw.get();
@@ -126,8 +123,11 @@ pub fn terminal_view(
         is_focused
     });
 
+    // for rust
+    let reg = regex::Regex::new("[\\w\\\\/-]+\\.(rs)?(toml)?:\\d+(:\\d+)?").unwrap();
+
     TerminalView {
-        data: ViewData::new(id),
+        id,
         term_id,
         raw: raw.get_untracked(),
         mode,
@@ -139,7 +139,7 @@ pub fn terminal_view(
         launch_error,
         internal_command,
         workspace,
-        hyper_matches: vec![],
+        hyper_regs: vec![reg],
         previous_mouse_action: Default::default(),
         current_mouse_action: Default::default(),
     }
@@ -169,29 +169,49 @@ impl TerminalView {
     }
 
     fn click(&self, pos: Point) -> Option<()> {
-        let raw_origin = self.raw.read();
+        let raw = self.raw.read();
         let position = self.get_terminal_point(pos);
-        let hy = self.hyper_matches.iter().find(|x| x.contains(&position))?;
-        let hyperlink = raw_origin.term.bounds_to_string(*hy.start(), *hy.end());
-        let content: Vec<&str> = hyperlink.split(':').collect();
-        let (file, line_str, col_str) =
-            (content.first()?, content.get(1)?, content.get(2)?);
-        let (line, col) =
-            (line_str.parse::<u32>().ok()?, col_str.parse::<u32>().ok()?);
-        let parent_path = self.workspace.path.as_ref()?;
-        self.internal_command.send(InternalCommand::JumpToLocation {
-            location: EditorLocation {
-                path: parent_path.join(file),
-                position: Some(EditorPosition::Position(Position::new(line, col))),
-                scroll_offset: None,
-                ignore_unconfirmed: false,
-                same_editor_tab: false,
-            },
-        });
+        let start_point = raw.term.semantic_search_left(position);
+        let end_point = raw.term.semantic_search_right(position);
+        let mut selection =
+            Selection::new(SelectionType::Simple, start_point, Side::Left);
+        selection.update(end_point, Side::Right);
+        selection.include_all();
+        if let Some(selection) = selection.to_range(&raw.term) {
+            let content = raw.term.bounds_to_string(selection.start, selection.end);
+            if let Some(match_str) =
+                self.hyper_regs.iter().find_map(|x| x.find(&content))
+            {
+                let hyperlink = match_str.as_str();
+                let content: Vec<&str> = hyperlink.split(':').collect();
+                let (file, line, col) = (
+                    content.first()?,
+                    content.get(1).and_then(|x: &&str| x.parse::<u32>().ok())?,
+                    content
+                        .get(2)
+                        .and_then(|x: &&str| x.parse::<u32>().ok())
+                        .unwrap_or(0),
+                );
+                let parent_path = self.workspace.path.as_ref()?;
+                self.internal_command.send(InternalCommand::JumpToLocation {
+                    location: EditorLocation {
+                        path: parent_path.join(file),
+                        position: Some(EditorPosition::Position(Position::new(
+                            line.saturating_sub(1),
+                            col.saturating_sub(1),
+                        ))),
+                        scroll_offset: None,
+                        ignore_unconfirmed: false,
+                        same_editor_tab: false,
+                    },
+                });
+                return Some(());
+            }
+        }
         None
     }
 
-    fn update_mouse_action_by_down(&mut self, mouse: PointerInputEvent) {
+    fn update_mouse_action_by_down(&mut self, mouse: &PointerInputEvent) {
         let mut next_action = MouseAction::None;
         match self.current_mouse_action {
             MouseAction::None
@@ -227,7 +247,8 @@ impl TerminalView {
         }
         self.current_mouse_action = next_action;
     }
-    fn update_mouse_action_by_up(&mut self, mouse: PointerInputEvent) {
+
+    fn update_mouse_action_by_up(&mut self, mouse: &PointerInputEvent) {
         let mut next_action = MouseAction::None;
         match self.current_mouse_action {
             MouseAction::None => {}
@@ -289,9 +310,11 @@ impl TerminalView {
     }
 
     fn get_terminal_point(&self, pos: Point) -> alacritty_terminal::index::Point {
+        let raw = self.raw.read();
         let col = (pos.x / self.char_size().width) as usize;
-        let line_no =
-            pos.y as i32 / (self.config.get().terminal_line_height() as i32);
+        let line_no = pos.y as i32
+            / (self.config.get().terminal_line_height() as i32)
+            - raw.term.grid().display_offset() as i32;
         alacritty_terminal::index::Point::new(
             alacritty_terminal::index::Line(line_no),
             alacritty_terminal::index::Column(col),
@@ -307,7 +330,6 @@ impl TerminalView {
         config: &LapceConfig,
     ) {
         let term_bg = config.color(LapceColor::TERMINAL_BACKGROUND);
-        let term_fg = config.color(LapceColor::TERMINAL_FOREGROUND);
 
         let font_size = config.terminal_font_size();
         let font_family = config.terminal_font_family();
@@ -335,9 +357,6 @@ impl TerminalView {
             let y =
                 (point.line.0 as f64 + content.display_offset as f64) * line_height;
             let char_y = y + (line_height - char_size.height) / 2.0;
-            let underline_y = (point.line.0 as f64 + content.display_offset as f64)
-                * line_height
-                + line_height;
             if y != line_content.y {
                 self.paint_line_content(
                     cx,
@@ -380,24 +399,6 @@ impl TerminalView {
                 }
             }
 
-            if self.hyper_matches.iter().any(|x| x.contains(&point)) {
-                let mut extend = false;
-                if let Some((_, end, _, _)) = line_content.underline.last_mut() {
-                    if *end == point.column.0 {
-                        *end += 1;
-                        extend = true;
-                    }
-                }
-                if !extend {
-                    line_content.underline.push((
-                        point.column.0,
-                        point.column.0 + 1,
-                        term_fg,
-                        underline_y,
-                    ));
-                }
-            }
-
             if cursor_point == &point {
                 line_content.cursor = Some((cell.c, x));
             }
@@ -485,33 +486,14 @@ impl Drop for TerminalView {
 }
 
 impl View for TerminalView {
-    fn view_data(&self) -> &ViewData {
-        &self.data
+    fn id(&self) -> ViewId {
+        self.id
     }
 
-    fn view_data_mut(&mut self) -> &mut ViewData {
-        &mut self.data
-    }
-
-    fn build(self) -> AnyWidget {
-        Box::new(self)
-    }
-}
-
-impl Widget for TerminalView {
-    fn view_data(&self) -> &ViewData {
-        &self.data
-    }
-
-    fn view_data_mut(&mut self) -> &mut ViewData {
-        &mut self.data
-    }
-
-    fn event(
+    fn event_before_children(
         &mut self,
         _cx: &mut EventCx,
-        _id_path: Option<&[Id]>,
-        event: Event,
+        event: &Event,
     ) -> EventPropagation {
         match event {
             Event::PointerDown(e) => {
@@ -523,7 +505,7 @@ impl Widget for TerminalView {
                 match self.current_mouse_action {
                     MouseAction::LeftOnce { pos, .. } => {
                         clear_selection = true;
-                        if self.click(pos).is_some() {
+                        if e.modifiers.control() && self.click(pos).is_some() {
                             return EventPropagation::Stop;
                         }
                     }
@@ -537,7 +519,7 @@ impl Widget for TerminalView {
                             .update(self.get_terminal_point(end_pos), Side::Right);
                         selection.include_all();
                         self.raw.write().term.selection = Some(selection);
-                        _cx.app_state_mut().request_paint(self.data.id());
+                        _cx.app_state_mut().request_paint(self.id);
                     }
                     MouseAction::LeftDouble { pos } => {
                         let position = self.get_terminal_point(pos);
@@ -553,7 +535,7 @@ impl Widget for TerminalView {
                         selection.update(end_point, Side::Right);
                         selection.include_all();
                         raw.term.selection = Some(selection);
-                        _cx.app_state_mut().request_paint(self.data.id());
+                        _cx.app_state_mut().request_paint(self.id);
                     }
                     MouseAction::RightOnce { pos } => {
                         let position = self.get_terminal_point(pos);
@@ -583,7 +565,7 @@ impl Widget for TerminalView {
                 }
                 if clear_selection {
                     self.raw.write().term.selection = None;
-                    _cx.app_state_mut().request_paint(self.data.id());
+                    _cx.app_state_mut().request_paint(self.id);
                 }
             }
             _ => {}
@@ -606,7 +588,7 @@ impl Widget for TerminalView {
                     self.raw = raw;
                 }
             }
-            cx.app_state_mut().request_paint(self.data.id());
+            cx.app_state_mut().request_paint(self.id);
         }
     }
 
@@ -614,14 +596,14 @@ impl Widget for TerminalView {
         &mut self,
         cx: &mut floem::context::LayoutCx,
     ) -> floem::taffy::prelude::NodeId {
-        cx.layout_node(self.data.id(), false, |_cx| Vec::new())
+        cx.layout_node(self.id, false, |_cx| Vec::new())
     }
 
     fn compute_layout(
         &mut self,
-        cx: &mut floem::context::ComputeLayoutCx,
+        _cx: &mut floem::context::ComputeLayoutCx,
     ) -> Option<Rect> {
-        let layout = cx.get_layout(self.data.id()).unwrap();
+        let layout = self.id.get_layout().unwrap_or_default();
         let size = layout.size;
         let size = Size::new(size.width as f64, size.height as f64);
         if size.is_empty() {
