@@ -1,9 +1,9 @@
 use floem::{
-    cosmic_text::{Attrs, AttrsList, LineHeightValue, TextLayout},
     peniko::kurbo::Rect,
     reactive::{create_effect, create_rw_signal},
-    view::View,
+    text::{Attrs, AttrsList, LineHeightValue, TextLayout},
     views::{container, label, rich_text, scroll, stack, Decorators},
+    View,
 };
 use lapce_core::buffer::rope_text::RopeText;
 
