@@ -159,3 +159,31 @@ fn listen_local_socket(proxy_rpc: ProxyRpcHandler) -> Result<()> {
     }
     Ok(())
 }
+
+pub fn get_url<T: reqwest::IntoUrl + Clone>(
+    url: T,
+    user_agent: Option<&str>,
+) -> Result<reqwest::blocking::Response> {
+    let mut builder = if let Ok(proxy) = std::env::var("https_proxy") {
+        let proxy = reqwest::Proxy::all(proxy)?;
+        reqwest::blocking::Client::builder()
+            .proxy(proxy)
+            .timeout(std::time::Duration::from_secs(10))
+    } else {
+        reqwest::blocking::Client::builder()
+            .timeout(std::time::Duration::from_secs(10))
+    };
+    if let Some(user_agent) = user_agent {
+        builder = builder.user_agent(user_agent);
+    }
+    let client = builder.build()?;
+    let mut try_time = 0;
+    loop {
+        let rs = client.get(url.clone()).send();
+        if rs.is_ok() || try_time > 3 {
+            return Ok(rs?);
+        } else {
+            try_time += 1;
+        }
+    }
+}
