@@ -108,8 +108,13 @@ impl PluginServerHandler for LspClient {
         self.host.handle_request(id, method, params, resp);
     }
 
-    fn handle_host_notification(&mut self, method: String, params: Params) {
-        let _ = self.host.handle_notification(method, params);
+    fn handle_host_notification(
+        &mut self,
+        method: String,
+        params: Params,
+        from: String,
+    ) {
+        let _ = self.host.handle_notification(method, params, from);
     }
 
     fn handle_did_save_text_document(
@@ -224,6 +229,7 @@ impl LspClient {
         let local_server_rpc = server_rpc.clone();
         let core_rpc = plugin_rpc.core_rpc.clone();
         let volt_id_closure = volt_id.clone();
+        let name = volt_display_name.clone();
         thread::spawn(move || {
             let mut reader = Box::new(BufReader::new(stdout));
             loop {
@@ -232,6 +238,7 @@ impl LspClient {
                         if let Some(resp) = handle_plugin_server_message(
                             &local_server_rpc,
                             &message_str,
+                            &name,
                         ) {
                             let _ = io_tx.send(resp);
                         }
